function [segment info] = regions(I)
% [S Info] = REGION(I)
% This function segmengt the incommoing image
% (2-D Matrix I) into connected regions, using 
% inequality of I values to determine borders of region
% 
% The return value S is of the same size than the input
% matrix I and contains a segment number within each 
% position.
%
% Info is a of size [no_of_Semgents, 5] and contains
% - segment value (in I)
% - segment size (no of positions belonging to this segm.)
% - segment center row (as a real number)
% - segment center column (as a real number)
% - segment index (going from 1 to no_of_segments)
% for each segment.
%

%%

neighbourShift = [ [0,1];[1,1];[1,0];[1,-1];[0,-1];[-1,-1];[-1,0];[-1,1]];
[h w]  = size(I);
segment = zeros(h,w,'int32');
count   = zeros(h*w,1,'int32');
census  = zeros(h*w,1,'int32');
center  = zeros(h*w,2);

segment_no = 0;
for y = 1:h
    for x = 1:w
        if (segment(y,x)>0)
            continue;
        else
            segment(y,x)=segment_no+1;
        end
        cz= I(y,x);
        census(segment_no+1)=cz;
        n = 1;
        list = [[y,x]];
        go_on = 1;
        while go_on>0
            n_last = n;
            n = 0;
            while n_last>0
              np = list(size(list,1)-n_last+1-n,:);
              nx = np(2);
              ny = np(1);
              for s = neighbourShift';
                  cx = nx+s(2);
                  cy = ny+s(1);
                  if ( cx>w || cx<1 || cy>h || cy<1)
                      continue;
                  end
                  if (I(cy,cx)==cz && segment(cy,cx)<1)
                      n = n + 1;
                      list = cat(1,list,np+s');
                      segment(cy,cx)=segment_no+1;
                  end
              end
              n_last = n_last - 1;
            end
            if n==0 ; go_on = 0; end                
        end
        segment_no = segment_no + 1;

        count(segment_no)=size(list,1);
        center(segment_no,:)=mean(list);
    end
end

count  = count(1:segment_no);
center = center(1:segment_no,:);
census = census(1:segment_no,:);

info = zeros(segment_no,5);
info(:,1)=census;
info(:,2)=count;
info(:,3)=center(:,1);
info(:,4)=center(:,2);
info(:,5)=1:segment_no;

end